/******************************************************************************
*     $Date: 2005/09/02 04:42:45 $
*     $RCSfile: main.c,v $
*******************************************************************************

*******************************************************************************
*
*     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
*     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
*     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
*     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
*     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
*     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
*     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
*     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
*     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
*     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
*     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
*     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
*     FOR A PARTICULAR PURPOSE.
*
*     (c) Copyright 2004 Xilinx, Inc.
*     All rights reserved.
*
*******************************************************************************/
/**
*     @file main.c
*
*     This file contains a simple example of how to set up and use the 
*     tri-mode ethernet peripheral on the GSRD system.
*     The program transmits a single buffer in a continuous loop. It sets up
*     a chain of buffer descriptors to send out the same buffer in a single
*     iteration.
*     It also sets up a chain of buffers to asynchronously handle received
*     ethernet frames.  It does not process the received data. It prints out
*     a "!" character for every 50000 received frames and a "@" character for
*     every 50000 transmitted frames.
*     This is a test program to verify that the system is set up right and 
*     working correctly. The GSRD system can be connected to a gigabit traffic
*     generator/measurement tool to verify frames are received and transmitted 
*     correctly. 
*     To enable jumbo frames, you would need to use the XLLTemac_SetOptions call 
*     right after the XLLTemac_Initialize call. You would need to appropriately 
*     increase the size of the receive buffers (if jumbo frames enabled on
*     receive)
* <pre>      
* MODIFICATION HISTORY:
*
*  Who  Date     Changes
* ---- -------- -----------------------------------------------
* sn  03/11/04 First release
*      
* </pre>
*
******************************************************************************/
#include "xparameters.h"
#include "xbasic_types.h"
#include "xio_dcr.h"
#include "xio.h"
#include "sleep.h"
#include "xcache_l.h"
#include "xlltdma.h"
#include "xlltemac.h"
#include "xpseudo_asm.h"
#include "xintc.h"
#include "xexception_l.h"
#include "xtime_l.h"
#include "stdio.h"
#include "topology.h"


void TxIntHandler(void *instance_p, XLLTBufDescriptor *bd_p, Xuint32 numbd);
void RxIntHandler(void *instance_p, XLLTBufDescriptor *bd_p, Xuint32 numbd);
void ErrorIntHandler(void *instance_p, XStatus stat );
XStatus SetupInterrupts(XIntc *Intc, Xuint32 deviceID, XLLTemac *instance_p );
void Timer_InterruptHandler(void *ptr);

#define INTERVAL 0x11E1A300  /* 1 sec with 300 Mhz clock */
#define POLL_INTERVAL 0x7530  /* 100 usec with 300 Mhz clock */
#if 0
#define POLL_INTERVAL 0x493e0  /* 1ms with 300 Mhz clock */
#endif

#define NUMTXBD 100
#define NUMRXBD 100
#define BUFFER_SIZE 9216 //receive and transmit frame buffer size

#define SOURCE_MAC0 0x00
#define SOURCE_MAC1 0x01
#define SOURCE_MAC2 0x02
#define SOURCE_MAC3 0x03
#define SOURCE_MAC4 0x04
#define SOURCE_MAC5 0x05

#define SOURCE_IP0 192
#define SOURCE_IP1 168
#define SOURCE_IP2 1
#define SOURCE_IP3 111

//source port 6902
#define SOURCE_PORT0 0x1A
#define SOURCE_PORT1 0xF6

#define DEST_MAC0 0x00
#define DEST_MAC1 0x0A
#define DEST_MAC2 0xE4
#define DEST_MAC3 0xB4
#define DEST_MAC4 0x03
#define DEST_MAC5 0x3C

#define DEST_IP0 192
#define DEST_IP1 168
#define DEST_IP2 1
#define DEST_IP3 100

//destination port 1334
#define DEST_PORT0 0x05
#define DEST_PORT1 0x36

//TCP/IP protocol
#define FRAME_TYPE0 0x00 
#define FRAME_TYPE1 0x21



unsigned char tx_buffer[1][BUFFER_SIZE] __attribute__ ((aligned (128))) = {
    {
      DEST_MAC0, DEST_MAC1, DEST_MAC2, DEST_MAC3, DEST_MAC4, DEST_MAC5,
			SOURCE_MAC0, SOURCE_MAC1, SOURCE_MAC2, SOURCE_MAC3, SOURCE_MAC4, SOURCE_MAC5,
			FRAME_TYPE0, FRAME_TYPE1,
			0x45, 0x00, 0x00, 0x2c, 0x6b, 0x2e, 0x00, 0x00, 0xfb, 0x06, 0xc0, 0xca,
			SOURCE_IP0, SOURCE_IP1, SOURCE_IP2, SOURCE_IP3,
			DEST_IP0, DEST_IP1, DEST_IP2, DEST_IP3,
			SOURCE_PORT0, SOURCE_PORT1,
			DEST_PORT0, DEST_PORT1,
			0x70, 0x09, 0x57, 0x19, 0x9b, 0x3d, 0x1a, 0x0e, 0x60, 0x12,
      0xfe, 0xf4, 0x00, 0x00, 0x00, 0x00, 0x02, 0x04, 0x05, 0x34, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
    }
  };

XLLTBufDescriptor txbd[NUMTXBD] __attribute__ ((aligned (128))) ;
XLLTBufDescriptor rxbd[NUMRXBD] __attribute__ ((aligned (128))) ;
unsigned char rx_buffer[NUMRXBD][BUFFER_SIZE] __attribute__ ((section (".noinit"),aligned (128)));
XLLTBufDescriptor localtxbd[NUMTXBD] __attribute__ ((aligned (128))) ;

//int globdebug = 1;

//#define SIMUL

#ifdef SIMUL
#define DBG2(x) 
#else
#define DBG2(x) x
#define DEBUG
#endif

volatile int dmaown_send_flag ;

volatile int send_frame_count = 0;
volatile int recv_frame_count = 0;
int timercnt = 0;
volatile int dummy, five_sec = 0;
volatile Xuint32 dummy2 = 0, dummy3 = 0;
volatile int test_started = 0, test_sec = 0;
volatile int cont_test = 1;

/*****************************************************************************/
/**
 * Main entry point to the  demo program 
 *
 * @param none
 *
 * @return 0
 *
 * @note 
 * none
 *
 * **************************************************************************/
int main()
{

  XLLTemac instance;   /* driver instance */
  XLLTDmaEngine txdma;  /* transmit DMA channel */
  XLLTDmaEngine rxdma;  /* receive DMA channel */
  XLLTBufDescriptor *bdptr; /* working pointer */
  static XIntc intc;           /* interrupt controller */
  XStatus status;
  Xint32  send_bytes;
	Xuint32 OptionsFlag;

  DBG2(xil_printf("\r\nMAIN instance_p %x\r\n", &instance);)
  dmaown_send_flag = 0;

#ifdef DCACHE_ON
  XCache_EnableDCache(0xffffffff);
#endif
  XCache_EnableICache(0xffffffff);

  status = XLLTemac_Initialize(&instance, XLLTEMAC_0, MyPhyNum);   /* device ID 0 */
  if (status){
    xil_printf("\r\nXLLTemac_Initialize error %x\r\n", status);
    return status;
  }

	xil_printf("\r\nSetting JUMBO frames flags\r\n");
	OptionsFlag = 0x00000003;
	status = XLLTemac_SetOptions(&instance, OptionsFlag, XLLTEMAC_0);
  if (status){
    xil_printf("\r\nJumbo frames EN in XLLTemac_SetOptions error %x\r\n", status);
    return status;
  }
  status = XLLTDmaEngine_Initialize(&txdma, XPAR_MPMC2_DDR_CDMAC_3_DCR_BASEADDR, 
      XLLT_CDMAC_TX0_ENGINE);
  if (status){
    xil_printf("\r\nXLLTDmaEngine_Initialize error %x\r\n", status);
    return status;
  }
  status = XLLTDmaEngine_Initialize(&rxdma, XPAR_MPMC2_DDR_CDMAC_3_DCR_BASEADDR, 
      XLLT_CDMAC_RX0_ENGINE);
  if (status){
    xil_printf("\r\nXLLTDmaEngine_Initialize error %x\r\n", status);
    return status;
  }
  
  XLLTSetDmaChannel(&instance, &txdma, &rxdma );
  XLLTemac_SetSgSendHandler(&instance, (void*)&instance,  TxIntHandler);
  XLLTemac_SetSgRecvHandler(&instance, (void *)&instance, RxIntHandler);
  XLLTemac_SetSgErrorHandler(&instance,(void *)&instance, ErrorIntHandler);

  XLLTemac_SetSgSendSpace(&instance, (Xuint32 *)&txbd[0], 
      sizeof(XLLTBufDescriptor) *  NUMTXBD) ;

  XLLTemac_SetSgRecvSpace(&instance, (Xuint32 *)&rxbd[0], 
      sizeof(XLLTBufDescriptor) *  NUMRXBD) ;

  /* set up interrupts */
  status = SetupInterrupts(&intc, XLLTEMAC_0, &instance);
  if (status){
    xil_printf("\r\nSetupInterrupts error %x\r\n", status);
    return status;
  }

  DBG2(xil_printf("SetupInterrupts done\r\n");)
  
	/* start up temac */
	status = XLLTemac_Start(&instance);
  if (status){
    xil_printf("\r\nUnable to start TEMAC error %x\r\n", status);
    return status;
  }

  /* initialize Tx buffer */
  for (dummy=64; dummy<BUFFER_SIZE; ++dummy ){
    tx_buffer[0][dummy] = (unsigned char)dummy;
  }

#ifdef DCACHE_ON
  for (dummy = 0; dummy < BUFFER_SIZE; dummy+=32 ){
    XCache_FlushDCacheLine((Xuint32)(&tx_buffer[0][dummy]));
  }
#endif

  dummy = 0;

  /* set up RX BD's */
  bdptr = &rxbd[0];
  for (dummy =0; dummy < NUMRXBD; ++dummy){
    bdptr->buf_p = (char *)&rx_buffer[dummy];
    bdptr->buf_len = BUFFER_SIZE;
#ifdef DCACHE_ON
  XCache_FlushDCacheLine((Xuint32)bdptr);
#endif
    bdptr = bdptr->next_p;
  }

  DBG2(xil_printf("[main]*****RxBd's******\r\n");)

  DBG2(XLLTdump_bd(&instance.RecvEngine);)
  XLLTemac_SgRecv(&instance, &rxbd[0]);
  DBG2(xil_printf("XLLTemac_SgRecv done\r\n");)

  DBG2(xil_printf("[main]*****TxBd's******\r\n");)
  DBG2(XLLTdump_bd(&instance.SendEngine);)
#ifdef DEBUG
  xil_printf("TX qeue head %x, tail %x, len %d\r\n", 
      &instance.SendEngine.queue.qhead_p, 
      &instance.SendEngine.queue.qtail_p, 
     instance.SendEngine.queue.qlen );
  xil_printf("SendEngine @ %x\r\n", &instance.SendEngine);
  xil_printf("RecvEngine @ %x\r\n", &instance.RecvEngine);
#endif
#define SENDBD 10
  while (cont_test){
		int i;
		if (test_started == 0){
			DBG2(xil_printf("Enter number of bytes to send\r\n");)
			#ifndef SIMUL
				scanf("%d", (int*) &send_bytes);
				send_bytes=-send_bytes;
			#else
				send_bytes = -BUFFER_SIZE;
			#endif
			if (send_bytes < 0){
				send_bytes = -send_bytes;
				xil_printf("Continuous test enabled\r\n");
			} else {
				cont_test = 0;
			}
			xil_printf("Sending %d bytes\r\n", send_bytes );
			test_started = 1;
		}
		
		if (test_started){
			/* set up BD's */
			for (i=0; i<SENDBD; i++){
				 bdptr = &localtxbd[i]; //setting current buffer descriptor
				 memset((char *)bdptr, 0, sizeof(*bdptr)); //setting ?
				 bdptr->next_p = &localtxbd[i+1]; //setting next buffer descriptor
				 bdptr->buf_p = &tx_buffer[0][0]; //setting data location
				 bdptr->buf_len = send_bytes; //setting number of send bytes
				 //start of packet and end of packet mask
				 bdptr->stat = XLLT_BDSTAT_SOP_MASK | XLLT_BDSTAT_EOP_MASK ; 
			}
			
			//fix the missing details
			localtxbd[SENDBD-1].next_p = 0; //no next pointer for the last bd
			localtxbd[SENDBD-1].stat = localtxbd[SENDBD-1].stat | 
							 XLLT_BDSTAT_INT_ON_END_MASK | 
							 XLLT_BDSTAT_STOP_ON_END_MASK; //trigger interrupt and stop on end
	
			XLLTemac_SgSend(&instance, &localtxbd[0], 0);
			//DBG2(XLLTdump_bd(&instance.SendEngine);) //DEBUG
		 }
		}
  return 0;

}

XStatus SetupInterrupts(XIntc *Intc, Xuint32 deviceID, XLLTemac *instance_p )
{
  
  XStatus Status;

  /*
   * Initialize the exception handler
   */
  XExc_Init();


  /*
   * Initialize the interrupt controller driver so that
   * it's ready to use, specify the device ID that is generated in
   * xparameters.h
   */
  Status = XIntc_Initialize(Intc, XPAR_DCR_INTC_0_DEVICE_ID);
  if (Status){
    xil_printf("XIntc_Initialize error %x\n", Status);
    return Status;
  }


  /*
   * Connect the Intc handler to the exception handler
   */
  XExc_RegisterHandler(XEXC_ID_NON_CRITICAL_INT,
                         (XExceptionHandler)XIntc_InterruptHandler,
                         (void *)Intc);
    
  /*
   * Connect a device driver handler that will be called when an interrupt
   * for the device occurs, the device driver handler performs the specifc
   * interrupt processing for the device
   */
  Status = XIntc_Connect(Intc, 
        XPAR_DCR_INTC_0_MPMC2_DDR_CDMAC_3_CDMAC_INT_INTR,
        (XInterruptHandler)XLLTIntrHandlerDma, instance_p);
  if (Status){
    xil_printf("XIntc_Connect error %x\n", Status);
    return Status;
  }
  


  /*
   * Start the interrupt controller such that interrupts are enabled for
   * all devices that cause interrupts, specific real mode so that
   * PS/2 can cause interrupt thru the interrupts controller.
   */
  Status = XIntc_Start(Intc, XIN_REAL_MODE);
  if (Status){
    xil_printf("XIntc_Start error %x\n", Status);
    return Status;
  }

  XIntc_Enable(Intc, XPAR_DCR_INTC_0_MPMC2_DDR_CDMAC_3_CDMAC_INT_INTR);

  /* 100 usec timer */
  XLLT_CDMAC_RX0_SETTIMER(XPAR_MPMC2_DDR_CDMAC_3_DCR_BASEADDR, 100);   
  XLLT_CDMAC_SET_INTR_REG(XPAR_MPMC2_DDR_CDMAC_3_DCR_BASEADDR, 
            XLLT_CMDAC_INT_MIE_MASK |  XLLT_CDMAC_INT_RX1_INT  );
#if 0
  /* PIT stuff */
  XExc_RegisterHandler(XEXC_ID_PIT_INT,
                         (XExceptionHandler)Timer_InterruptHandler,
                         instance_p );
    

  XTime_PITSetInterval(INTERVAL);
  XTime_TSRClearStatusBits(XREG_TSR_CLEAR_ALL );
  /* XTime_PITEnableAutoReload(); */
  XTime_PITEnableInterrupt();
#endif
  /*
   * Enable non-critical exceptions
   */
  XExc_mEnableExceptions(XEXC_ALL);


  return 0 ;
}


/**************************************************************************/
/**
 * Transmit complete interrupt handler. This callback function is called when 
 * one or more ethernet frames have been successfully transmitted. 
 * Any upper protocol layer processing can be done here ( for example, 
 * freeing socket buffer data structures associated with each buffer
 * descriptor). Note that the app5 field in the buffer descriptor can be
 * used to store protocol specific information before the XLLTemac_SgSend call
 * is made. So for instance, the socket buffer structure can be stored in the
 * app5 field prior to the transmit , and the structure can be freed in this
 * handler routine after pulling it out of the app5 field in the buffer 
 * descriptor.
 *
 * @param instance_p pointer to the XLLTemac instance
 * @param bd_p points to the head of a chain of buffer descriptors sent out
 * @numbd number of buffers sent out
 *
 * @note 
 * none
 * *************************************************************************/

void TxIntHandler(void *instance_p, XLLTBufDescriptor *bd_p, Xuint32 numbd)
{

  DBG(xil_printf("TxIntHandler: instance_p %x, bd_p %x, numbd %d\r\n", 
          instance_p, bd_p, numbd);)

  send_frame_count+=numbd;
  if (send_frame_count >50000){
    xil_printf("@");
    send_frame_count = 0;
  }

}

/**************************************************************************/
/**
 * Receive callback function which is called by the interrupt handler
 * when one or more ethernet frames are received.
 * note that in this example, we are not processing the buffers associated
 * with the buffer descriptors. So we're saying its ok to re-use the same
 * buffer for future received packets. Realistically we would have to 
 * attach a new buffer to the buffer descriptor before we return from this
 * routine
 *
 * @param instance_p pointer to the XLLTemac instance
 * @param bd_p pointer to the head of a chain of buffer descriptors
 * @param numbd number of buffer descriptors pointing to received buffers.
 *
 * @note 
 * none
 */
void RxIntHandler(void *instance_p, XLLTBufDescriptor *bd_p, Xuint32 numbd)
{

  DBG(xil_printf("RxIntHandler: instance_p %x, bd_p %x, numbd %d\r\n", 
        instance_p, bd_p, numbd);)

  recv_frame_count+=numbd;
  if (recv_frame_count > 50000) {
    xil_printf("!");
    recv_frame_count = 0;
  }

}

/**************************************************************************/
/**
 *
 * Error interrupt handler routine. This should never be called. There is 
 * no recovery when this error happens
 *
 * @param instance_p pointer to XLLTemac instance
 * stat   any error status - currently not used.
 *
 * ***********************************************************************/

void ErrorIntHandler(void *instance_p, XStatus stat)
{

  xil_printf("ErrorIntHandler %x\n", stat);
  while (1) ;
}


/**************************************************************************/
/**
 *
 * Timer interrupt handler routine. This should be periodically called
 * at a frequency as high as possible. This polls the receive buffers for
 * any data. This is necessary if interrupt coalescing feature is used.
 * Please refer to the software driver documentation for the need
 * of this timer.
 * @param ptr pointer to XLLTemac instance
 *
 * @note 
 * none
 *
 * ***********************************************************************/
void Timer_InterruptHandler(void *ptr)
{

  XLLTemac *instance_p = (XLLTemac *)ptr;

  XLLTprocess_rx(instance_p, TIMER_INT_MASK);

  XTime_PITClearInterrupt();
  XTime_PITSetInterval(POLL_INTERVAL);
  timercnt++;
  return;

}

